/*
 * (C) Copyright 2003-2004
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2004
 * Mark Jonas, Freescale Semiconductor, mark.jonas@freescale.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <mpc5xxx.h>
#include <pci.h>
#include <i2c.h>

#include "sdram.h"

#include "mt46v32m16tg-75.h"

#include <asm/mmu.h>
#include <asm/processor.h>


#if defined(CONFIG_INIT_MMU)

static __inline__ unsigned long get_msr (void) {
  unsigned long msr;
  asm volatile("mfmsr %0" : "=r" (msr) :);
  return msr;
};

static __inline__ void set_msr (unsigned long msr) {
  asm volatile("mtmsr %0" : : "r" (msr));
};


void init_mmu(void) {
  // enable BAT4-BAT7
  mtspr(HID2,HID2_HBE);
  // SDRAM: 128M at 0x0 cached
  write_bat(DBAT0,0x00000ffe,0x00000002);
  write_bat(IBAT0,0x00000ffe,0x00000002);
  // MBAR: 16M at 0xf0000000 uncached
  write_bat(DBAT1,0xf00001fe,0xf000002a);
  write_bat(IBAT1,0xf00001fe,0xf000002a);
  // FLASH: 64M at 0xfc000000 data uncached, instructions cached
  write_bat(DBAT2,0xfc0007fe,0xfc00002a);
  write_bat(IBAT2,0xfc0007fe,0xfc000002);
  // PCI MEM: 256M at 0x40000000 uncached
  write_bat(DBAT3,0x40001ffe,0x4000002a);
  write_bat(IBAT3,0x40001ffe,0x4000002a);
  // PCI IO: 16M at 0x50000000 uncached
  write_bat(DBAT4,0x500001fe,0x5000002a);
  write_bat(IBAT4,0x500001fe,0x5000002a);
  // FLASH: 64M at 0xf8000000 cached
  write_bat(DBAT5,0xf80007fe,0xfc000002);
  write_bat(IBAT5,0xf80007fe,0xfc000002);
  // SDRAM: 128M at 0xc0000000 cached
  write_bat(DBAT6,0xc0000ffe,0x00000002);
  write_bat(IBAT6,0xc0000ffe,0x00000002);
  // enable BAT
  set_msr(get_msr()|(1<<4)|(1<<5));
};
#endif

long int initdram_real (int board_type)
{
	sdram_conf_t sdram_conf;

	sdram_conf.ddr = SDRAM_DDR;
	sdram_conf.mode = SDRAM_MODE;
	sdram_conf.emode = SDRAM_EMODE;
	sdram_conf.control = SDRAM_CONTROL;
	sdram_conf.config1 = SDRAM_CONFIG1;
	sdram_conf.config2 = SDRAM_CONFIG2;
	sdram_conf.tapdelay = SDRAM_TAPDELAY;
	return mpc5xxx_sdram_init (&sdram_conf);
}

#ifdef CONFIG_BOARD_EARLY_DRAMINIT
long int initdram_early(int board_type)
{
  DECLARE_GLOBAL_DATA_PTR;
  return gd->ram_size;
};
#endif

long int initdram (int board_type)
{
#ifdef CONFIG_BOARD_EARLY_DRAMINIT
  return initdram_early (board_type);
#else
  return initdram_real (board_type);
#endif
};

#ifdef CONFIG_BOARD_EARLY_DRAMINIT
int board_early_draminit(void)
{
    DECLARE_GLOBAL_DATA_PTR;
	gd->ram_size=initdram_real(0);
#if defined(CONFIG_USE_ICACHE)
	icache_enable();
#endif
#if defined(CONFIG_INIT_MMU)
	init_mmu();
#if defined(CONFIG_USE_DCACHE)
	dcache_enable();
#endif
#endif
	return 0;
};
#endif

int checkboard (void)
{
	puts ("Board: Media5200 ");

/*
 * Retrieve FPGA Revision.
 */
printf ("(FPGA %08X)\n", *(vu_long *) (CFG_FPGA_BASE + 0x400));

	return 0;
}



/** get ethernet address from EEPROM if non in the env **/
void board_get_enetaddr (uchar *addr)
{
	char ethaddr[20];
	
	/* Initialize I2C */
	i2c_init (CFG_I2C_SPEED, CFG_I2C_SLAVE);

	/* Read the 6 bytes containing the MAC at offset of 0x00 */
	i2c_read (CFG_I2C_EEPROM_ADDR, CFG_EEPROM_MACADDR_OFFSET, CFG_I2C_EEPROM_ADDR_LEN, addr, 6);
	
	sprintf (ethaddr, "%02x:%02x:%02x:%02x:%02x:%02x",
		addr[0], addr[1], addr[2], addr[3], addr[4], addr[5]);
		
	printf ("*** Using default MAC address from the EEPROM: %s ***\n\n", ethaddr);
	setenv ("ethaddr", ethaddr);
}

#if defined(CONFIG_PCI)
static struct pci_controller hose;

extern void pci_mpc5xxx_init(struct pci_controller *);

void pci_init_board(void)
{
	pci_mpc5xxx_init(&hose);
}

#endif /* defined(CONFIG_PCI) */

#if defined (CFG_CMD_IDE) && defined (CONFIG_IDE_RESET)
void init_ide_reset (void)
{
	debug ("init_ide_reset\n");

	/* assert reset */
	*(vu_long *) (CFG_FPGA_BASE + 0x480) &= ~0x08000000UL;

	/* disable level shifters and enable level shifter power supply */
	*(vu_long *) (CFG_FPGA_BASE + 0x480) |= 0x06000000UL;

	/* enable level shifters */
	*(vu_long *) (CFG_FPGA_BASE + 0x480) &= ~0x04000000UL;
}

void ide_set_reset (int idereset)
{
	debug ("ide_reset(%d)\n", idereset);

	if (idereset) {
		*(vu_long *) (CFG_FPGA_BASE + 0x480) &= ~0x08000000UL;
	} else {
		*(vu_long *) (CFG_FPGA_BASE + 0x480) |= 0x08000000UL;
	}
}
#endif /* defined (CFG_CMD_IDE) && defined (CONFIG_IDE_RESET) */


#ifdef CONFIG_CONSOLE_EXTRA_INFO
/* Return text to be printed besides the logo. */
void video_get_info_str (int line_number, char *info)
{
	if (line_number == 1) {
		strcpy (info, " Media5200");
	} else {
		info [0] = '\0';
	}
}
#endif


int misc_init_r (void)
{
	/* Enable audio paths in the Media5200 FPGA */
	*(vu_long *) (CFG_FPGA_BASE + 0x490) = 0UL;
}
