/*
 * (C) Copyright 2003
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2004,2006 Freescale Semiconductor Inc
 * Author: Bernhard Kuhn <bkuhn@freescale.com> [BK]
 *
 * BK: BTCS sample interface implementation
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Boot support
 */
#include <common.h>
#include <linux/string.h>
#include <command.h>
#include <asm/io.h>
#include <mpc5xxx.h>

#include <btcs_defs.h>

extern int btcs_copy(void);
extern int btcs_install(void);
extern int btcs_remove(void);

#ifdef CFG_BTCS
#if (CONFIG_COMMANDS & CFG_CMD_BTCS)

btcs_cb_interface_t* btcs_interface = (btcs_cb_interface_t*) CFG_BTCS_CB_DST;

static int btcs_ossignal(btcs_event_t* event) {
	switch(event->cmd) {
		case BTCS_REG_MODIFIED:
			printf("[%i] %08x\n",(int)event->arg,btcs_interface->shm->reg[(int)event->arg]);
			break;
	};
	return 0;
};

int do_btcs ( cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	DECLARE_GLOBAL_DATA_PTR;

	if (argc < 2) {
		printf("btcs [s|d <register>|m <register> <value>|c|i|r]\n");
		printf("  s = show all registers\n");
		printf("  d = display a specific register\n");
		printf("  m = modifiy a specific register\n");
		printf("  o = observe register changes\n");
		printf("  c = copy isr and cb from flash to ram\n");
		printf("  i = install\n");
		printf("  r = remove\n");
		return 0;
	}

	if ((strcmp(argv[1], "c") == 0) && (argc == 2)) {
		int ret = btcs_copy();
		switch(ret) {
			case -1:
				printf("btcs ISR magic not found at 0x%08x\n",CFG_BTCS_ISR_SRC);
				break;
			case -2:
				printf("btcs CB magic not found at 0x%08\n",CFG_BTCS_CB_SRC);
				break;
			case -3:
				printf("btcs already installed\n");
				break;
		};
		return 0;
	};

	if ((strcmp(argv[1], "i") == 0) && (argc == 2)) {
		int ret = btcs_install();
		switch(ret) {
			case -1:
				printf("btcs ISR magic not found at 0x%08x\n",CFG_BTCS_ISR_DST);
				break;
			case -2:
				printf("btcs CB magic not found at 0x%08\n",CFG_BTCS_CB_DST);
				break;
			case -3:
				printf("btcs already installed\n");
				break;
		};
		return 0;
	};

	if ((strcmp(argv[1], "r") == 0) && (argc == 2)) {
		int ret = btcs_remove();
		if(ret==-3) {
			printf("btcs not installed\n");
		};
		return 0;
	};

	if(!gd->btcs_installed) {
		printf("btcs callback handler not installed\n");
		return 0;
	};

	if ((strcmp(argv[1], "o") == 0) && (argc == 2)) {
		// observer mode
		btcs_interface->ossignal = btcs_ossignal;
		while(!ctrlc()) {
			// do nothing
		};
		btcs_interface->ossignal = 0;
	}
	else if ((strcmp(argv[1], "s") == 0) && (argc == 2)) {
		// show all
		int i;
		for(i=0;i<BTCS_NUM_REGS;i++) {
			printf("[%i] %08x\n",i,btcs_interface->shm->reg[i]);
		};
	}
	else if ((strcmp(argv[1], "d") == 0) && (argc == 3)) {
		// display one
		int num = simple_strtoul(argv[2],0,10);
		if((num>=0)&&(num<BTCS_NUM_REGS)) {
			printf("[%i] %08x\n",num,btcs_interface->shm->reg[num]);
		}
		else {
			printf("invalid register number: %i (register range: 0 - %i)\n",
					num,BTCS_NUM_REGS-1);
		};
	}
	else if ((strcmp(argv[1], "m") == 0) && (argc == 4)) {
		// modify one
		int num = simple_strtoul(argv[2],0,10);
		int val = simple_strtoul(argv[3],0,16);
		if((num>=0)&&(num<BTCS_NUM_REGS)) {
			extern int btcs_callback(btcs_event_t *event);
			btcs_event_t event;
			btcs_interface->shm->reg[num] = val;
			event.cmd = BTCS_REG_MODIFIED;
			event.arg = (void*) num;
			btcs_callback(&event);
		}
		else {
			printf("invalid register number: %i (register range: 0 - %i)\n",
					num,BTCS_NUM_REGS-1);
		};
	}
	else {
		printf("syntax error\n");
		return 0;
	}

	return 0;
}




/* -------------------------------------------------------------------- */

U_BOOT_CMD(
		btcs,	CFG_MAXARGS,	1,	do_btcs,
		"btcs  - btcs command\n",
		NULL
	  );
#endif	/* CFG_CMD_BTCS */
#endif  /* CFG_BTCS */
