/*
 * (C) Copyright 2005 Metrowerks, Inc.
 *
 * Author: Bernhard Kuhn <bkuhn@metrowerks.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
/* Video support for Fujitsu CoralPA chipset                               */

#include <common.h>

#ifdef CONFIG_VIDEO_CORALPA

#include <video_fb.h>
#include <coralpa.h>
#include <pci.h>



extern inline int in_8(volatile unsigned char *addr)
{
	int ret;

	__asm__ __volatile__(
		"lbz%U1%X1 %0,%1;\n"
		"twi 0,%0,0;\n"
		"isync" : "=r" (ret) : "m" (*addr));
	return ret;
}

extern inline void out_8(volatile unsigned char *addr, int val)
{
	__asm__ __volatile__("stb%U0%X0 %1,%0; eieio" : "=m" (*addr) : "r" (val));
}

extern inline int in_le16(volatile unsigned short *addr)
{
	int ret;

	__asm__ __volatile__("lhbrx %0,0,%1;\n"
			     "twi 0,%0,0;\n"
			     "isync" : "=r" (ret) :
			      "r" (addr), "m" (*addr));
	return ret;
}

extern inline int in_be16(volatile unsigned short *addr)
{
	int ret;

	__asm__ __volatile__("lhz%U1%X1 %0,%1;\n"
			     "twi 0,%0,0;\n"
			     "isync" : "=r" (ret) : "m" (*addr));
	return ret;
}

extern inline void out_le16(volatile unsigned short *addr, int val)
{
	__asm__ __volatile__("sthbrx %1,0,%2; eieio" : "=m" (*addr) :
			      "r" (val), "r" (addr));
}

extern inline void out_be16(volatile unsigned short *addr, int val)
{
	__asm__ __volatile__("sth%U0%X0 %1,%0; eieio" : "=m" (*addr) : "r" (val));
}

extern inline unsigned in_le32(volatile unsigned *addr)
{
	unsigned ret;

	__asm__ __volatile__("lwbrx %0,0,%1;\n"
			     "twi 0,%0,0;\n"
			     "isync" : "=r" (ret) :
			     "r" (addr), "m" (*addr));
	return ret;
}

extern inline unsigned in_be32(volatile unsigned *addr)
{
	unsigned ret;

	__asm__ __volatile__("lwz%U1%X1 %0,%1;\n"
			     "twi 0,%0,0;\n"
			     "isync" : "=r" (ret) : "m" (*addr));
	return ret;
}

extern inline void out_le32(volatile unsigned *addr, int val)
{
	__asm__ __volatile__("stwbrx %1,0,%2; eieio" : "=m" (*addr) :
			     "r" (val), "r" (addr));
}

extern inline void out_be32(volatile unsigned *addr, int val)
{
	__asm__ __volatile__("stw%U0%X0 %1,%0; eieio" : "=m" (*addr) : "r" (val));
}


#define inb(port)               in_8((u8 *)(port))
#define outb(val, port)         out_8((u8 *)(port), (val))
#define inw(port)               in_le16((u16 *)(port))
#define outw(val, port)         out_le16((u16 *)(port), (val))
#define inl(port)               in_le32((u32 *)(port))
#define outl(val, port)         out_le32((u32 *)(port), (val))


static unsigned long baseaddr;

static void coralpa_i2c_write(int slave_address, int sub_address, int data) {

  /* activate bus */
  outl(0x00,CORALPA_I2C_ADR);
  outl(0x3f,CORALPA_I2C_CCR);
  outl(0x00,CORALPA_I2C_BCR);

  // write slave address
  outl(slave_address&0xfe,CORALPA_I2C_DAR);
  outl(0x10,CORALPA_I2C_BCR);
  while((inl(CORALPA_I2C_BSR)&0x01)!=0x01);
  if((inl(CORALPA_I2C_BSR)&0x10)==0x10) {
    printf("error while writing slave address\n");
    return;
  };

  // write sub-address
  outl(sub_address,CORALPA_I2C_DAR);
  outl(0x10,CORALPA_I2C_BCR);
  while((inl(CORALPA_I2C_BSR)&0x01)!=0x01);
  if((inl(CORALPA_I2C_BSR)&0x10)==0x10) {
    printf("error while writing sub-address address\n");
    return;
  };

  // write data
  outl(data,CORALPA_I2C_DAR);
  outl(0x10,CORALPA_I2C_BSR);
  while((inl(CORALPA_I2C_BSR)&0x01)!=0x01);
  if((inl(CORALPA_I2C_BSR)&0x10)==0x10) {
    printf("error while writing data\n");
    return;
  };

  /* de-activate bus */
  outl(0x00,CORALPA_I2C_BCR);
  outl(0x00,CORALPA_I2C_CCR);
  outl(0x00,CORALPA_I2C_ADR);

};

/* It is unclear if this function is actually doing anything yet */
#if 0
static void coralpa_i2c_write_lcd(int slave_address, int data) {

  /* activate bus */
  outl(0x00,CORALPA_I2C_ADR);
  outl(0x3f,CORALPA_I2C_CCR);
  outl(0x00,CORALPA_I2C_BCR);

  // write slave address
  outl(slave_address&0xfe,CORALPA_I2C_DAR);
  outl(0x10,CORALPA_I2C_BCR);
  while((inl(CORALPA_I2C_BSR)&0x01)!=0x01);
  if((inl(CORALPA_I2C_BSR)&0x10)==0x10) {
    printf("error while writing slave address\n");
    return;
  };

  // write data
  outl(data,CORALPA_I2C_DAR);
  outl(0x10,CORALPA_I2C_BSR);
  while((inl(CORALPA_I2C_BSR)&0x01)!=0x01);
  if((inl(CORALPA_I2C_BSR)&0x10)==0x10) {
    printf("error while writing data\n");
    return;
  };

  /* de-activate bus */
  outl(0x00,CORALPA_I2C_BCR);
  outl(0x00,CORALPA_I2C_CCR);
  outl(0x00,CORALPA_I2C_ADR);

};
#endif

void coralpa_SiI164_init(void) {
  /* get SiI164 out of reset */

  outl(0x00004000,CORALPA_IOM); // SB pin is output
  outl(0x00000080|0x00800000,CORALPA_GD); // set SB pin high

  /* write init sequence */
  coralpa_i2c_write(0x70,0x08,0x35);
  coralpa_i2c_write(0x70,0x09,0x09);
};


void coralpa_LCD_init(void) {
  /* enable LVDS and backlight for LCD */

  outl(0x00001280,CORALPA_IOM); // enable outputs on BC, EDO, and ECK pins
  
  /* BC pin = Backlight_On# = 0
     ECK pin = LVDS_Powerdown# = 1
     EDO pin = FrameRateCntrl = 1
  */
  outl(0x00000005|0x00250000,CORALPA_GD);

  /* It is unclear if this function is actually doing anything yet. */
  
  /* Turn up the brightness */
  //coralpa_i2c_write_lcd(0x5C,0x7f);
  
};




#if 0
/* Original Settings 1024x768 */
static int hpixels       = 1024;
static int hsyncstart    = 1032;
static int hsyncstop     = 1152;
static int htotal        = 1360;
static int vpixels       =  768;
static int vsyncstart    =  784;
static int vsyncstop     =  787;
static int vtotal        =  823;
static int bytesperpixel =    2;
static int devider       =    4;
#endif

#if 1
/* 1024x768 from Coral P */
static int hpixels       = 1024;
static int hsyncstart    = 1048;
static int hsyncstop     = 1183;
static int htotal        = 1389;
static int vpixels       =  768;
static int vsyncstart    =  771;
static int vsyncstop     =  776;
static int vtotal        =  806;
static int bytesperpixel =    2;
static int devider       =    5;
#endif

#if 0
/* 640x480 */
static int hpixels       =  640;
static int hsyncstart    =  656;
static int hsyncstop     =  752;
static int htotal        =  800;
static int vpixels       =  480;
static int vsyncstart    =  490;
static int vsyncstop     =  492;
static int vtotal        =  525;
static int bytesperpixel =    2;
static int devider       =   15;
#endif



void coralpa_init(unsigned long addr) {

  baseaddr = addr;

  // outw(CORALPA_FRST_FRST,CORALPA_FRST);
  // udelay(1000);
  // outw(CORALPA_SRST_SRST,CORALPA_SRST);
  // udelay(1000);

  outl(CORALPA_CCF_COT133|CORALPA_CCF_CGE166,CORALPA_CCF);

  udelay(1000);

  /* Initialize SDRAM */
  outl(CORALPA_MMR_CL3|
       CORALPA_MMR_SAW_14BANK2ROW12COL9|
       CORALPA_MMR_RTS_64N(4)|
       CORALPA_MMR_LOWD|
       CORALPA_MMR_TRCD(3)|
       CORALPA_MMR_TRAS(7)|
       CORALPA_MMR_TRP(3)|
       CORALPA_MMR_TRC(10)|
       CORALPA_MMR_TRRD(3)|
       CORALPA_MMR_TWR(1),
       CORALPA_MMR);
  
  udelay(1000);

  /* Setup horizontal timings */
  outw(hpixels-1,CORALPA_HDP);
  outw(hpixels-1,CORALPA_HDB);
  outw(hsyncstart-1,CORALPA_HSP);
  outb(hsyncstop-hsyncstart-1,CORALPA_HSW);
  outw(htotal-1,CORALPA_HTP);

  /* Setup vertical timings */
  outw(vpixels-1,CORALPA_VDP);
  outw(vsyncstart-1,CORALPA_VSP);
  outb(vsyncstop-vsyncstart-1,CORALPA_VSW);
  outw(vtotal-1,CORALPA_VTR);

  /* Setup memory location */
  outl(0,CORALPA_L0OA);
  outl(0,CORALPA_L0DA);
  outw(0,CORALPA_L0DX);
  outw(0,CORALPA_L0DY);

  /* Setup mode */
  outl(((bytesperpixel==2)?CORALPA_L0M_L0C:0)|
       CORALPA_L0M_L0W(bytesperpixel*hpixels/64)|
       CORALPA_L0M_L0H(vpixels-1),
       CORALPA_L0M);
  outl(0,CORALPA_L0EM);

  /* Setup clock and enable layer 0 */
  outw(CORALPA_DCME_SC(devider),CORALPA_DCME);
  outw(CORALPA_DCE_DEN|CORALPA_DCE_L0E,CORALPA_DCE);

  // coralpa_SiI164_init();
	coralpa_LCD_init();
};


static struct pci_device_id supported[] = {
        { 0x10cf, 0x201e },
        { 0x10cf, 0x2019 },
        { }
};

GraphicDevice coralpa;

void *video_hw_init (void)
{
    unsigned int *vm, i;
    pci_dev_t devno;
    u32 iobase, size;
    int idx = 0;

    iobase = 0;
    size = htotal*vtotal*bytesperpixel;

    while (1) {
      /* Find PCI device
       */
      if ((devno = pci_find_devices (supported, idx++)) < 0) {
	break;
      }
      
      pci_read_config_dword (devno, PCI_BASE_ADDRESS_0, &iobase);
      iobase &= ~0xf;

      printf("GFX:   CoralP at 0x%08x\n",iobase);

      coralpa_init(iobase);

      /* Clear video memory */
      i = size/4;
      vm = (unsigned int *)iobase;
      while(i--)
	*vm++ = 0;
    };


    if(iobase) {
      memset (&coralpa, 0, sizeof (GraphicDevice));

      coralpa.winSizeX = hpixels;
      coralpa.winSizeY = vpixels;
      
#if defined(CONFIG_VIDEO_CORALPA_16BPP)
      coralpa.gdfIndex = GDF_15BIT_555RGB;
      coralpa.gdfBytesPP = bytesperpixel;
      
#else
#error Unsupported CORALPA BPP
#endif
      
      coralpa.memSize = coralpa.winSizeX * coralpa.winSizeY * coralpa.gdfBytesPP;
      coralpa.frameAdrs = iobase;
    
      return (&coralpa);
    }
    else {
      return -1;
    };
}

void video_set_lut (
    unsigned int index,           /* color number */
    unsigned char r,              /* red */
    unsigned char g,              /* green */
    unsigned char b               /* blue */
    )
{
}

#endif
