/*
 * (C) Copyright 2003-2004
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2004-2005
 * Mark Jonas, Freescale Semiconductor, mark.jonas@freescale.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef __CONFIG_H
#define __CONFIG_H


/*
 * High Level Configuration Options
 * (easy to change)
 */

#define CONFIG_MPC5xxx		1	/* This is an MPC5xxx CPU */
#define CONFIG_MPC5200		1	/* ... MPC5200 to be exact */
#define CONFIG_MEDIA5200	1	/* ... on Media5200 board */

#define CFG_MPC5XXX_CLKIN	33000000 /* ... running at 33.000000MHz */

#define BOOTFLAG_COLD		0x01	/* Normal Power-On: Boot from FLASH  */
#define BOOTFLAG_WARM		0x02	/* Software reboot	     */

#define CFG_CACHELINE_SIZE	32	/* For MPC5xxx CPUs */
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#  define CFG_CACHELINE_SHIFT	5	/* log base 2 of the above value */
#endif

/*
 * Serial console configuration
 */
#define CONFIG_PSC_CONSOLE	6	/* console is on PSC6 */
#define CONFIG_BAUDRATE		115200	/* ... at 115200 bps */
#define CFG_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200, 230400 }

/*
 * Video console
 */
#if 1
#define CONFIG_VIDEO
#define CONFIG_VIDEO_CORALPA
#define CONFIG_VIDEO_CORALPA_16BPP

#define CONFIG_CFB_CONSOLE
#define CONFIG_VIDEO_LOGO
#define CONFIG_VIDEO_BMP_LOGO
#define CONFIG_CONSOLE_EXTRA_INFO
#define CONFIG_VGA_AS_SINGLE_DEVICE
#define CONFIG_VIDEO_SW_CURSOR
#define CONFIG_SPLASH_SCREEN

#define ADD_VIDEO_CMD	CFG_CMD_BMP

/* #define for 1024x768x16 found in videomodes.c */
#define CFG_DEFAULT_VIDEO_MODE 0x317

#else
#define ADD_VIDEO_CMD	0
#endif

/*
 * PCI Mapping:
 * 0x40000000 - 0x4fffffff - PCI Memory
 * 0x50000000 - 0x50ffffff - PCI IO Space
 */
#define CONFIG_PCI		1
#define CONFIG_PCI_PNP		1
#define CONFIG_PCI_SCAN_SHOW	1

#define CONFIG_PCI_MEM_BUS	0x40000000
#define CONFIG_PCI_MEM_PHYS	CONFIG_PCI_MEM_BUS
#define CONFIG_PCI_MEM_SIZE	0x10000000

#define CONFIG_PCI_IO_BUS	0x50000000
#define CONFIG_PCI_IO_PHYS	CONFIG_PCI_IO_BUS
#define CONFIG_PCI_IO_SIZE	0x01000000

#define CONFIG_NET_MULTI	1  /* support more than one network adapter */
#define CONFIG_MII		1
/* Intel PRO/100 PCI network card support */
#define CONFIG_EEPRO100		1
#define CFG_RX_ETH_BUFFER	8  /* use 8 rx buffer on eepro100  */
/* NatSemi DP8382x PCI network card support */
#define CONFIG_NS8382X		1
/* The RTL8139 driver detects the card but does not work.
 * Additionally, the driver could need some cleanup that on non-MIPS platforms
 * the KSEG1ADDR macro and on all platforms the _IO_BASE define are not needed.
#define CONFIG_RTL8139		1
#define _IO_BASE		0x50000000
#define KSEG1ADDR(a)		(a)
*/

#define ADD_PCI_CMD 		CFG_CMD_PCI

/* Partitions */
#define CONFIG_MAC_PARTITION
#define CONFIG_DOS_PARTITION

/* USB */
#if 1
#define CONFIG_USB_OHCI
#define ADD_USB_CMD             CFG_CMD_USB | CFG_CMD_FAT
#define CONFIG_USB_STORAGE
#else
#define ADD_USB_CMD             0
#endif

/* early dram init and fixups for MPC5200 B3 */
#if 1
#define CONFIG_BOARD_EARLY_DRAMINIT	1
#define CFG_RCR_DELAY_SETTINGS		1
#if 0
#define CFG_SLV_DELAY_SETTINGS		1 /* for MPC5200B2 only*/
#endif
#endif


#if 1
#define CFG_XLB_PIPELINING	1
#define CFG_PCI_SNOOPING        1
#define CONFIG_PHY_LATECHECK    1

#endif
/* CACHE support */
#if 1
#define CONFIG_INIT_MMU 1
#define CONFIG_USE_ICACHE 1
//#define CONFIG_USE_DCACHE 1
#define ADD_CMD_CACHE 		CFG_CMD_CACHE
#endif


/*
 * Supported commands
 */
#define CONFIG_COMMANDS		(CONFIG_CMD_DFL	| \
				 CFG_CMD_PING	| \
				 CFG_CMD_I2C	| \
				 CFG_CMD_EEPROM	| \
				 CFG_CMD_FAT	| \
				 CFG_CMD_IDE	| \
				 ADD_VIDEO_CMD  | \
				 ADD_PCI_CMD	| \
				 CFG_CMD_DHCP	| \
				 CFG_CMD_REGINFO| \
				 ADD_CMD_CACHE  | \
				 ADD_USB_CMD)

/* this must be included AFTER the definition of CONFIG_COMMANDS (if any) */
#include <cmd_confdefs.h>

#if (TEXT_BASE == 0xFE000000)		/* Boot low */
#   define CFG_LOWBOOT		1
#endif

/*
 * Autobooting
 */
#define CONFIG_BOOTDELAY	5	/* autoboot after 5 seconds */
#define CONFIG_ZERO_BOOTDELAY_CHECK	/* check for keypress on bootdelay==0 */

#define CONFIG_PREBOOT	\
	"usb reset; setenv stdout serial;"			\
	"setenv stderr serial;echo;"				\
	"echo 'Autostarting. Press any key to abort...';echo"

#undef	CONFIG_BOOTARGS

#define	CONFIG_EXTRA_ENV_SETTINGS					\
	"skipnetcheck=y\0"							\
	"skipidecheck=y\0"							\
	"netdev=eth0\0"							\
	"autoload=no\0"							\
	"autostart=no\0"						\
	"script_addr=0x200000\0"					\
	"install_usb=setenv script bootrc.img;run script_usb\0"		\
	"script_usb=run usbstart;"					\
		"fatload usb 0:${usbpart} ${script_addr} ${script};"	\
		"autoscr ${script_addr}\0"				\
	"usbstart=usb reset; usb storage\0"				\
	"usbpart=1\0"							\
	"reset_env=erase 0xfff80000 0xfffbffff\0"			\
	""

#define CONFIG_BOOTCOMMAND	"run install_usb"

/*
 * IPB Bus clocking configuration.
 */
#define CFG_IPBSPEED_133   		/* define for 133MHz speed */

/*
 * I2C configuration
 */
#define CONFIG_HARD_I2C		1	/* I2C with hardware support */
#define CFG_I2C_MODULE		2	/* Select I2C module #1 or #2 */

#define CFG_I2C_SPEED		100000 /* 100 kHz */
#define CFG_I2C_SLAVE		0x7F

/*
 * EEPROM configuration
 */
#define CFG_I2C_EEPROM_ADDR		0x50	/* 1010000x */
#define CFG_I2C_EEPROM_ADDR_LEN		1
#define CFG_EEPROM_PAGE_WRITE_BITS	3
#define CFG_EEPROM_PAGE_WRITE_DELAY_MS	70
#define CFG_EEPROM_PRODUCT_INFO_OFFSET  0x00    /* counting from the beginning of the eeprom */
#define CFG_EEPROM_MACADDR_OFFSET	0x10    /* counting from the beginning of the eeprom */

/*
 * Flash configuration
 */
#define CFG_FLASH_CFI		1	/* Flash is CFI conformant */
#define CFG_FLASH_CFI_DRIVER	1	/* Use the common driver */
#define CFG_MAX_FLASH_BANKS	2	/* max num of flash banks */
#define CFG_FLASH_BANKS_LIST { CFG_CS1_START, CFG_BOOTCS_START }
#define CFG_FLASH_EMPTY_INFO
#define CFG_MAX_FLASH_SECT	256	/* max num of sects on one chip */


#define CFG_FLASH_NO_BUFF_WRITE_CP_STATUS  /* allows progress report on per sect basis during cp */

#define CFG_FLASH_BASE	0xFC000000

/* allow 4 128K sectors for u-boot before ENV starts */
#if defined(CFG_LOWBOOT)
#   define CFG_ENV_ADDR		0xFE080000
#else	/* CFG_LOWBOOT */
#   define CFG_ENV_ADDR		0xFFF80000
#endif	/* CFG_LOWBOOT */

/*
 * Environment settings
 */
#define CFG_ENV_IS_IN_FLASH	1
#define CFG_ENV_SIZE		0x40000
#define CFG_ENV_SECT_SIZE	0x40000
#define CONFIG_ENV_OVERWRITE	1

/*
 * Memory map
 */
#define CFG_SDRAM_BASE		0x00000000
#define CFG_SDRAM_SIZE_CS0	0x08000000	/*  128 MB */
#define CFG_SDRAM_SIZE_CS1	0x00000000	/*    0 MB */
#define CFG_SDRAM_HI_ADDR	1
#define CFG_DEFAULT_MBAR	0x80000000
#define CFG_MBAR		0xF0000000	/*   64 kB */
#define CFG_FPGA_BASE		0xF0010000	/*   64 kB */
#define CFG_DUART_BASE		0xF0020000	/*   64 kB */

/* Use SRAM until RAM will be available */
#define CFG_INIT_RAM_ADDR	MPC5XXX_SRAM
#define CFG_INIT_RAM_END	MPC5XXX_SRAM_SIZE	/* End of used area in DPRAM */

#define CFG_GBL_DATA_SIZE	128	/* size in bytes reserved for initial data */
#define CFG_GBL_DATA_OFFSET	(CFG_INIT_RAM_END - CFG_GBL_DATA_SIZE)
#define CFG_INIT_SP_OFFSET	CFG_GBL_DATA_OFFSET

#define CFG_MONITOR_BASE    TEXT_BASE
#if (CFG_MONITOR_BASE < CFG_FLASH_BASE)
#   define CFG_RAMBOOT		1
#endif

#define CFG_MONITOR_LEN		(192 << 10)	/* Reserve 192 kB for Monitor	*/
#define CFG_MALLOC_LEN		(128 << 10)	/* Reserve 128 kB for malloc()	*/
#define CFG_BOOTMAPSZ		(8 << 20)	/* Initial Memory map for Linux */

/*
 * Ethernet configuration
 */
#define CONFIG_MPC5xxx_FEC	1
#define CONFIG_FEC_MII100	1
#define CONFIG_PHY_ADDR		0x00

/*
 * GPIO configuration
 *
 * CS1:   SDRAM CS1 disabled, gpio_wkup_6 enabled                0
 * Reserved                                                      0
 * ALTs:  CAN1 on I2C1, CAN2 on Tmr0/1                           01
 * CS7:   Interrupt GPIO on PSC3_5                               0
 * CS8:   Interrupt GPIO on PSC3_4                               0
 * ATA:   reset default, changed in ATA driver                   00
 * IR_USB_CLK: IrDA/USB 48MHz clock gen. int., pin is GPIO       0
 * IRDA:  UART (PSC6) / IrDA                                     101
 * ETHER: reset default, changed in Ethernet driver              0000
 * PCI_DIS: reset default, changed in PCI driver                 0
 * USB_SE: reset default, changed in USB driver                  0
 * USB:   reset default, changed in USB driver                   00
 * PSC3:  SPI and UART functionality without CD                  1100
 * Reserved                                                      0
 * PSC2:  reset default, changed in AC'97 driver                 000
 * Reserved                                                      0
 * PSC1:  reset default, changed in I2S driver                   000
 *
 */
#define CFG_GPS_PORT_CONFIG	0x10500C00

/*
 * Miscellaneous configurable options
 */
#define CFG_LONGHELP			/* undef to save memory	    */
#define CFG_PROMPT		"=> "	/* Monitor Command Prompt   */
#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define CFG_CBSIZE		1024	/* Console I/O Buffer Size  */
#else
#define CFG_CBSIZE		256	/* Console I/O Buffer Size  */
#endif
#define CFG_PBSIZE (CFG_CBSIZE+sizeof(CFG_PROMPT)+16)	/* Print Buffer Size */
#define CFG_MAXARGS		16		/* max number of command args	*/
#define CFG_BARGSIZE		CFG_CBSIZE	/* Boot Argument Buffer Size	*/

#define CFG_MEMTEST_START	0x00100000	/* memtest works on */
#define CFG_MEMTEST_END		0x00f00000	/* 1 ... 15 MB in DRAM	*/

#define CFG_LOAD_ADDR		0x100000	/* default load address */

#define CFG_HZ			1000	/* decrementer freq: 1 ms ticks */

/*
 * Various low-level settings
 */
#define CFG_HID0_INIT		HID0_ICE | HID0_ICFI
#define CFG_HID0_FINAL		HID0_ICE

#define CFG_BOOTCS_START	(CFG_CS1_START + CFG_CS1_SIZE)
#define CFG_BOOTCS_SIZE		0x02000000	/* 32 MB */
#define CFG_BOOTCS_CFG		0x0008DF00	/* 8WS, MX, AL, CE, AS_25, DS_32 */
#define CFG_CS1_START		CFG_FLASH_BASE
#define CFG_CS1_SIZE		0x02000000	/* 32 MB */
#define CFG_CS1_CFG		0x0008DF00	/* 8WS, MX, AL, CE, AS_25, DS_32 */

#define CFG_CS2_START		CFG_FPGA_BASE
#define CFG_CS2_SIZE		0x00010000	/* 64 kB */
#define CFG_CS2_CFG		0x0006FF00	/* 6WS, MX, AL, AA, CE, AS_25, DS_32 */

#define CFG_CS3_START		CFG_DUART_BASE
#define CFG_CS3_SIZE		0x00010000	/* 64 kB */
#define CFG_CS3_CFG		0x00085800	/* 8WS, CE, AS_24, DS_8 */

#define CFG_CS_BURST		0x00000000
#define CFG_CS_DEADCYCLE	0x33333333

/*-----------------------------------------------------------------------
 * USB stuff
 *-----------------------------------------------------------------------
 */
#define CONFIG_USB_CLOCK	0x0001BBBB
#define CONFIG_USB_CONFIG	0x00001000

/*-----------------------------------------------------------------------
 * IDE/ATA stuff Supports IDE harddisk
 *-----------------------------------------------------------------------
 */

#undef  CONFIG_IDE_8xx_PCCARD		/* Use IDE with PC Card	Adapter	*/

#undef	CONFIG_IDE_8xx_DIRECT		/* Direct IDE    not supported	*/
#undef	CONFIG_IDE_LED			/* LED   for ide not supported	*/

#define	CONFIG_IDE_RESET		/* reset for ide supported	*/
#define CONFIG_IDE_PREINIT

#define CFG_IDE_MAXBUS		1	/* max. 1 IDE bus		*/
#define CFG_IDE_MAXDEVICE	1	/* max. 1 drive per IDE bus	*/

#define CFG_ATA_IDE0_OFFSET	0x0000

#define CFG_ATA_BASE_ADDR	MPC5XXX_ATA

/* Offset for data I/O			*/
#define CFG_ATA_DATA_OFFSET	(0x0060)

/* Offset for normal register accesses	*/
#define CFG_ATA_REG_OFFSET	(CFG_ATA_DATA_OFFSET)

/* Offset for alternate registers	*/
#define CFG_ATA_ALT_OFFSET	(0x005C)

/* Interval between registers	*/
#define CFG_ATA_STRIDE          4

/* Boot time critical services settings */
#ifdef CFG_BTCS
#define CFG_BTCS_ISR_SRC        (CFG_FLASH_BASE + 0x03FC0000)	/* last flash sector */
#define CFG_BTCS_ISR_DST        0xC0400000	/* 4M */
#define CFG_BTCS_ISR_SIZE       0x8000		/* 32K */
#define CFG_BTCS_CB_SRC         (CFG_FLASH_BASE + 0x03FC1000)
#define CFG_BTCS_CB_DST         0xC0410000	/* 4M + 64K */
#define CFG_BTCS_CB_SIZE        0x8000		/* 32K */
#define CFG_BTCS_BENCHMARK      1
#endif


#endif /* __CONFIG_H */
