/*
 * (C) Copyright 2005 Metrowerks, Inc.
 *
 * Author: Bernhard Kuhn <bkuhn@metrowerks.com>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef CORALPA_H
#define CORALPA_H

/* CoralP register definitions */

#define PCI_VENDOR_ID_FUJITSUSIEMENS (0x10cf)
#define PCI_DEVICE_ID_FUJITSUSIEMENS_CORALPA (0x2019)

#define CORALPA_HOSTBASE                 (baseaddr+0x1fc0000)
#define CORALPA_SRST                     (CORALPA_HOSTBASE+0x002c)
#define CORALPA_SRST_SRST                ((0x1)<< 0)
#define CORALPA_FRST                     (CORALPA_HOSTBASE+0x00a0)
#define CORALPA_FRST_FRST                ((0x1)<< 0)
#define CORALPA_CCF                      (CORALPA_HOSTBASE+0x0038)
#define CORALPA_CCF_COT100               ((0x0)<<16)
#define CORALPA_CCF_COT133               ((0x1)<<16)
#define CORALPA_CCF_CGE100               ((0x0)<<18)
#define CORALPA_CCF_CGE133               ((0x1)<<18)
#define CORALPA_CCF_CGE166               ((0x2)<<18)
#define CORALPA_CID                      (CORALPA_HOSTBASE+0x00f0)
#define CORALPA_MMR                      (CORALPA_HOSTBASE+0xfffc)
#define CORALPA_MMR_CL2                  ((0x2)<< 0)
#define CORALPA_MMR_CL3                  ((0x3)<< 0)
#define CORALPA_MMR_ASW64                ((0x1)<< 3)
#define CORALPA_MMR_SAW_15BANK2ROW13COL9 ((0x1)<< 4)
#define CORALPA_MMR_SAW_14BANK2ROW12COL9 ((0x7)<< 4)
#define CORALPA_MMR_SAW_14BANK2ROW12COL8 ((0x6)<< 4)
#define CORALPA_MMR_SAW_13BANK2ROW11COL8 ((0x5)<< 4)
#define CORALPA_MMR_SAW_12BANK1ROW11COL8 ((0x4)<< 4)
#define CORALPA_MMR_RTS_384              ((0x0)<< 7)
#define CORALPA_MMR_RTS_1552             ((0x7)<< 7)
#define CORALPA_MMR_RTS_64N(n)           (( n )<< 7)
#define CORALPA_MMR_LOWD                 ((0x2)<<10)
#define CORALPA_MMR_TRCD(x)              (( x )<<12)
#define CORALPA_MMR_TRAS(x)              (( x )<<14)
#define CORALPA_MMR_TRP(x)               (( x )<<17)
#define CORALPA_MMR_TRC(x)               (( x )<<19)
#define CORALPA_MMR_TRRD(x)              (( x )<<23)
#define CORALPA_MMR_TWR(x)               ((x-1)<<30)
#define CORALPA_IOM                      (CORALPA_HOSTBASE+0x00a8)
#define CORALPA_GD                       (CORALPA_HOSTBASE+0x00ac)
#define CORALPA_DISPLAYBASE              (baseaddr+0x1fd0000)
#define CORALPA_DCM                      (CORALPA_DISPLAYBASE+0x000)
#define CORALPA_DCM_SYNC_INTERLACE       ((0x2)<< 0)
#define CORALPA_DCM_SYNC_INTERLACEVIDEO  ((0x3)<< 0)
#define CORALPA_DCM_ESY                  ((0x1)<< 2)
#define CORALPA_DCM_SF                   ((0x1)<< 3)
#define CORALPA_DCM_EEQ                  ((0x1)<< 7)
#define CORALPA_DCM_SC(x)                (( x )<< 8)
#define CORALPA_DCM_CKS                  ((0x1)<<15)
#define CORALPA_DCE                      (CORALPA_DISPLAYBASE+0x002)
#define CORALPA_DCE_L0E                  ((0x1)<< 0)
#define CORALPA_DCE_L1E                  ((0x1)<< 1)
#define CORALPA_DCE_L2E                  ((0x1)<< 2)
#define CORALPA_DCE_L3E                  ((0x1)<< 3)
#define CORALPA_DCE_DEN                  ((0x1)<<15)
#define CORALPA_HTP                      (CORALPA_DISPLAYBASE+0x006)
#define CORALPA_HDP                      (CORALPA_DISPLAYBASE+0x008)
#define CORALPA_HDB                      (CORALPA_DISPLAYBASE+0x00a)
#define CORALPA_HSP                      (CORALPA_DISPLAYBASE+0x00c)
#define CORALPA_HSW                      (CORALPA_DISPLAYBASE+0x00e)
#define CORALPA_VSW                      (CORALPA_DISPLAYBASE+0x00f)
#define CORALPA_VTR                      (CORALPA_DISPLAYBASE+0x012)
#define CORALPA_VSP                      (CORALPA_DISPLAYBASE+0x014)
#define CORALPA_VDP                      (CORALPA_DISPLAYBASE+0x016)
#define CORALPA_WX                       (CORALPA_DISPLAYBASE+0x018)
#define CORALPA_WY                       (CORALPA_DISPLAYBASE+0x01a)
#define CORALPA_WW                       (CORALPA_DISPLAYBASE+0x01c)
#define CORALPA_WH                       (CORALPA_DISPLAYBASE+0x01e)
#define CORALPA_L0M                      (CORALPA_DISPLAYBASE+0x020)
#define CORALPA_L0M_L0H(x)               (( x )<< 0)
#define CORALPA_L0M_L0W(x)               (( x )<<16)
#define CORALPA_L0M_L0C                  ((0x1)<<31)
#define CORALPA_L0OA                     (CORALPA_DISPLAYBASE+0x024)
#define CORALPA_L0DA                     (CORALPA_DISPLAYBASE+0x028)
#define CORALPA_L0DX                     (CORALPA_DISPLAYBASE+0x02c)
#define CORALPA_L0DY                     (CORALPA_DISPLAYBASE+0x02e)
#define CORALPA_DCME                     (CORALPA_DISPLAYBASE+0x100)
#define CORALPA_DCME_SYNC_INTERLACE      ((0x2)<< 0)
#define CORALPA_DCME_SYNC_INTERLACEVIDEO ((0x3)<< 0)
#define CORALPA_DCME_ESY                 ((0x1)<< 2)
#define CORALPA_DCME_SF                  ((0x1)<< 3)
#define CORALPA_DCME_EEQ                 ((0x1)<< 7)
#define CORALPA_DCME_SC(x)               (( x )<< 8)
#define CORALPA_DCME_CKS                 ((0x1)<<15)
#define CORALPA_DCEE                     (CORALPA_DISPLAYBASE+0x102)
#define CORALPA_DCEE_L0E                 ((0x1)<< 0)
#define CORALPA_DCEE_L1E                 ((0x1)<< 1)
#define CORALPA_DCEE_L2E                 ((0x1)<< 2)
#define CORALPA_DCEE_L3E                 ((0x1)<< 3)
#define CORALPA_DCEE_L4E                 ((0x1)<< 4)
#define CORALPA_DCEE_L5E                 ((0x1)<< 5)
#define CORALPA_DCEE_DEN                 ((0x1)<<15)
#define CORALPA_L0EM                     (CORALPA_DISPLAYBASE+0x110)
#define CORALPA_L0EM_L0WP                ((0x1)<< 0)
#define CORALPA_L0EM_L0PB(x)             (( x )<<20)
#define CORALPA_L0EM_L0EC_DC             ((0x1)<<30)
#define CORALPA_L0WX                     (CORALPA_DISPLAYBASE+0x114)
#define CORALPA_L0WY                     (CORALPA_DISPLAYBASE+0x116)
#define CORALPA_L0WW                     (CORALPA_DISPLAYBASE+0x118)
#define CORALPA_L0WH                     (CORALPA_DISPLAYBASE+0x11a)
#define CORALPA_DBGC                     (CORALPA_DISPLAYBASE+0x184)
#define CORALPA_DBGC_DBGR(r)             (( r )<<16)
#define CORALPA_DBGC_DBGG(g)             (( g )<< 8)
#define CORALPA_DBGC_DBGB(b)             (( b )<< 0)
#define CORALPA_L0PAL                    (CORALPA_DISPLAYBASE+0x400)
#define CORELP_L0PAL_SET(r,g,b,a)        ((a<<24)|(r<<16)|(g<<8)|(b<<0))
#define CORALPA_I2CBASE                  (baseaddr+0x1fcc000)
#define CORALPA_I2C_BSR                  (CORALPA_I2CBASE+0x000)
#define CORALPA_I2C_BCR                  (CORALPA_I2CBASE+0x004)
#define CORALPA_I2C_CCR                  (CORALPA_I2CBASE+0x008)
#define CORALPA_I2C_ADR                  (CORALPA_I2CBASE+0x00c)
#define CORALPA_I2C_DAR                  (CORALPA_I2CBASE+0x010)

#endif
