/*
 * (C) Copyright 2004
 * Metrowerks
 * Author: Bernhard Kuhn <bkuhn@metrowerks.com> 
 *
 * (C) Copyright 2004
 * Metrowerks, Author: Bernhard Kuhn <bkuhn@metrowerks.com> [BK]
 *
 * BK: added BTCS support
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <mpc5xxx.h>
#include <asm/processor.h>

#ifdef CFG_BTCS

#include <btcs_defs.h>

static __inline__ unsigned long get_msr (void) {
  unsigned long msr;
  asm volatile("mfmsr %0" : "=r" (msr) :);
  return msr;
};

static __inline__ void set_msr (unsigned long msr) {
  asm volatile("mtmsr %0" : : "r" (msr));
};



int btcs_copy(void) {
	DECLARE_GLOBAL_DATA_PTR;

	btcs_isr_interface_t* isr_src = (btcs_isr_interface_t*) (CFG_BTCS_ISR_SRC);
	btcs_cb_interface_t* cb_src = (btcs_cb_interface_t*) (CFG_BTCS_CB_SRC);

	int isr_magic = isr_src->magic;
	int cb_magic = cb_src->magic;

	int isr_size = isr_src->size;
	int cb_size = cb_src->size;;

	// setup btcs only if there is a valid magic
	if(isr_magic!=0xDEADBEEF) {
		return -1;
	};

	if(cb_magic!=0xDEADBEEF) {
		return -2;
	};

	// don't copy if already installed
	if(gd->btcs_installed==1) return -3;

	// copy cb
	memcpy((void *)(CFG_BTCS_CB_DST-0xc0000000),(void *)CFG_BTCS_CB_SRC,cb_size);
	flushDCache(CFG_BTCS_CB_DST-0xc0000000,cb_size);
	invalidateICache(CFG_BTCS_CB_DST-0xc0000000,cb_size);
	invalidateDCache(CFG_BTCS_CB_DST,cb_size);
	invalidateICache(CFG_BTCS_CB_DST,cb_size);

	// copy isr
	memcpy((void *)(CFG_BTCS_ISR_DST-0xc0000000),(void *)CFG_BTCS_ISR_SRC,isr_size);
	flushDCache(CFG_BTCS_ISR_DST-0xc0000000,isr_size);
	invalidateICache(CFG_BTCS_ISR_DST-0xc0000000,isr_size);
	invalidateDCache(CFG_BTCS_ISR_DST,isr_size);
	invalidateICache(CFG_BTCS_ISR_DST,isr_size);

	return 0;
};

int btcs_install(void) {
	DECLARE_GLOBAL_DATA_PTR;

	btcs_isr_interface_t* isr_dst = (btcs_isr_interface_t*) (CFG_BTCS_ISR_DST);
	btcs_cb_interface_t* cb_dst = (btcs_cb_interface_t*) (CFG_BTCS_CB_DST);

	void (*btcs_code)(void);

	// install btcs only if there is a valid magic
	if(isr_dst->magic!=0xDEADBEEF) {
		return -1;
	};
	if(cb_dst->magic!=0xDEADBEEF) {
		return -2;
	};

	// don't install if already installed
	if(gd->btcs_installed==1) return -3;

	// cb setup
	btcs_code=(void(*)(void))(cb_dst->setup);
	btcs_code();

	// isr init
	btcs_code=(void(*)(void))(isr_dst->start);
	btcs_code();

	// cb start
	btcs_code=(void(*)(void))(cb_dst->start);
	btcs_code();

	gd->btcs_installed=1;

	return 0;
};

/*
 * save the kernel kernel isr vector 
 */
int btcs_save_kernel_isr(void) {
	DECLARE_GLOBAL_DATA_PTR;

	btcs_isr_interface_t* isr_dst = (btcs_isr_interface_t*) (CFG_BTCS_ISR_DST);
	btcs_cb_interface_t* cb_dst = (btcs_cb_interface_t*) (CFG_BTCS_CB_DST);

	void (*btcs_code)(void);

	// install btcs only if there is a valid magic
	if(isr_dst->magic!=0xDEADBEEF) {
		return -1;
	};
	if(cb_dst->magic!=0xDEADBEEF) {
		return -2;
	};

	// can't reinstall if not already installed
	if(gd->btcs_installed==0) return -3;

	// isr reinit init
	btcs_code=(void(*)(void))(isr_dst->save_kernel_isr);
	btcs_code();

	return 0;
};

/*
 * reinstall the bcts isr vector after
 * it is trashed by loading the kernel
 */
int btcs_restore_btcs_isr(void) {
	DECLARE_GLOBAL_DATA_PTR;

	btcs_isr_interface_t* isr_dst = (btcs_isr_interface_t*) (CFG_BTCS_ISR_DST);
	btcs_cb_interface_t* cb_dst = (btcs_cb_interface_t*) (CFG_BTCS_CB_DST);

	void (*btcs_code)(void);

	// install btcs only if there is a valid magic
	if(isr_dst->magic!=0xDEADBEEF) {
		return -1;
	};
	if(cb_dst->magic!=0xDEADBEEF) {
		return -2;
	};

	// can't reinstall if not already installed
	if(gd->btcs_installed==0) return -3;

	// isr reinit init
	btcs_code=(void(*)(void))(isr_dst->restore_btcs_isr);
	btcs_code();

	return 0;
};



int btcs_remove(void) {
	DECLARE_GLOBAL_DATA_PTR;

	btcs_isr_interface_t* isr_dst = (btcs_isr_interface_t*) (CFG_BTCS_ISR_DST);
	btcs_cb_interface_t* cb_dst = (btcs_cb_interface_t*) (CFG_BTCS_CB_DST);

	void (*btcs_code)(void);

	// don't uninstall if not installed
	if(gd->btcs_installed==0) return -3;

	// cb stop
	btcs_code=(void(*)(void))(cb_dst->stop);
	btcs_code();

	// isr exit
	btcs_code=(void(*)(void))(isr_dst->stop);
	btcs_code();

	// cb cleanup
	btcs_code=(void(*)(void))(cb_dst->cleanup);
	btcs_code();

	gd->btcs_installed=0;

	return 0;
};


int btcs_callback(btcs_event_t *event) {
	int retval;
	btcs_cb_interface_t* btcs;
	btcs = (btcs_cb_interface_t*) CFG_BTCS_CB_DST;
	int (*callback)(btcs_event_t*);
	callback = btcs->callback;
	set_msr( get_msr() & (~MSR_EE));
	retval = callback(event);
	set_msr( get_msr() | MSR_EE);
	return retval;
};

void btcs_setup(void) {
	DECLARE_GLOBAL_DATA_PTR;

	if(btcs_copy()!=0) return;
	btcs_install();
#ifdef CFG_BTCS_BENCHMARK
	gd->btcs_benchmark=*((volatile unsigned int*)0xf0000708);
#endif
};

#endif /* CFG_BTCS */
